/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * END COPYRIGHT BLOCK **/
package com.netscape.management.admserv;

import java.net.URL;
import java.net.MalformedURLException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.StringTokenizer;
import netscape.ldap.LDAPConnection;
import netscape.ldap.LDAPEntry;
import netscape.ldap.LDAPAttribute;
import netscape.ldap.LDAPException;
import com.netscape.management.nmclf.*;
import com.netscape.management.client.console.ConsoleInfo;
import com.netscape.management.client.topology.IProductObject;
//import com.netscape.management.client.topology.KeyCertMigrationDialog;
import com.netscape.management.client.util.Debug;
import com.netscape.management.client.util.AdmTask;
import com.netscape.management.client.util.LDAPUtil;
import com.netscape.management.client.util.URLByteEncoder;
import com.netscape.management.client.util.ResourceSet;
import com.netscape.management.client.util.UtilConsoleGlobals;




/**
 * Netscape admin server migration entry point. The directory server
 * needs to contain the name of this class in order for the topology
 * view to load this class for migrating the admin server.
 *
 * @author   phlee
 * @date     01/09/98
 */

public class AdminServerProduct implements IProductObject {

    protected ConsoleInfo _consoleInfo; // global information
    //protected KeyCertMigrationDialog _keyCert;


    /**
     * constructor.
     */
    public AdminServerProduct() {
    }


    /*public static void main(String args[]) {
     AdminServerProduct asp = new AdminServerProduct();
         ConsoleInfo consoleInfo = null;
         try {
             LDAPConnection connection = new LDAPConnection();
             connection.connect("awing", 3000, "uid=admin, ou=Administrators, ou=TopologyManagement, o=NetscapeRoot", "admin");
             consoleInfo = new ConsoleInfo("buddha.mcom.com", 8081, "admin", "admin", "ou=mcom.com, o=NetscapeRoot");
             consoleInfo.setAdminURL("http://buddha.mcom.com:8081/");
             consoleInfo.setBaseDN("cn=admin-serv-buddha, ou=mcom.com, o=NetscapeRoot");
             consoleInfo.setCurrentDN("cn=admin-serv-buddha, ou=mcom.com, o=NetscapeRoot");
      consoleInfo.setLDAPConnection(connection);
         } catch (Exception e) {System.out.println(e); e.printStackTrace();}
     asp.initialize(consoleInfo);
         asp.migrate("c:\\netscape", "c:\\tmp", "cn=Server Group, cn=buddha.mcom.com, ou=mcom.com, o=NetscapeRoot", false);
     }*/

    /**
     * Implements the IProductObject interface. Initializes the page with
     * the global information.
     *
     * @param info - global information.
     */
    public void initialize(ConsoleInfo info) {
        _consoleInfo = info;
    }


    /**
      * Implements the IProductObject interface. Starts the Admin Server
      * migration process. Since there can only be one admin server per
      * admin group, migration always results in the overwriting of the
      * existing SIE. Also, since the admin server that is being invoked
      * to do the migration via URL belongs to the target config root,
      * the targetDN is also unused. Moreover, the server parameter is not
      * needed since there can only be one admin server per server root.
      *
      * @param  serverRoot - directory path for the migration origin.
      * @param  server - the server to migrate from the serverRoot.
      * @param  targetDN - the destination admin group DN.
      * @param  overwrite - whether to overwrite existing SIE.
      * @return  boolean value indicating whether the process succeeded (true)
      *          or failed (false).
      */
    public boolean migrate(String serverRoot, String server,
            String targetDN, boolean overwrite) {

        //ResourceSet resource = new ResourceSet("com.netscape.management.admserv.admserv");
        ResourceSet resource = AdminServer._resource;
        String adminURL = _consoleInfo.getAdminURL();
        if (adminURL == null || adminURL.equals("")) {
            Debug.println("ERROR AdminServerProduct.migrate: admin URL is not available.");
            return false;
        }

        if (migrateConfig(adminURL, serverRoot) == false) {
            Debug.println("ERROR AdminServerProduct.migrate: migrateConfig failed.");
            String msg = resource.getString("status", "migrateConfigFailed");
            String title = resource.getString("status", "migrateTitle");
            SuiOptionPane.showMessageDialog(null, msg, title,
                    SuiOptionPane.ERROR_MESSAGE);
            return false;
        }

        /*String commandString = new String(adminURL + "admin-serv/tasks/configuration/ListAlias");
        Hashtable args = new Hashtable();
        args.put("serverRoot", serverRoot);
        args.put("serverVer" , "3");

        Vector aliasList = new Vector();
        try {
            URL command = new URL(commandString);
            AdmTask task = new AdmTask(command,
                    _consoleInfo.getAuthenticationDN(),
                    _consoleInfo.getAuthenticationPassword());
            task.setArguments(args);
            int execStatus = task.exec();
            if (execStatus != 0) {
                Debug.println("ERROR AdminServerProduct.migrateKeyCert: exec failed");
                return false;
            } else {
                String list = (String)(task.getResult("aliasList"));
                StringTokenizer st = new StringTokenizer(list, ",", false);
                while (st.hasMoreTokens()) {
                    aliasList.addElement(st.nextToken());
                }
            }
        } catch (Exception e) {
            Debug.println("AdminServerProduct:"+e);
        }

        _keyCert = new KeyCertMigrationDialog(
                UtilConsoleGlobals.getActivatedFrame(), _consoleInfo,
                serverRoot, aliasList, server,
                getTargetAdminServerID(targetDN), "");
        if (_keyCert == null) {
            Debug.println("ERROR AdminServerProduct.migrate: could not create KeyCertMigrationDialog.");
            return false;
        }*/
        /*String newAlias = getTargetAdminServerID(targetDN);
         if (newAlias == null) {
             return false;
         }
          Absolute path of file containing the absolute path to key/cert
         String adminConfigFile = serverRoot + "/admin-serv/config/ns-admin.conf";
         _keyCert.setConfigFile(adminConfigFile);
         _keyCert.setNewAlias(newAlias);
         _keyCert.setPassword("");*/
        /*_keyCert.show();
        if (_keyCert.isCancel()) {
            Debug.println("TRACE AdminServerProduct.migrate: skipping migrateKeyCert");
        }*/
        /*else {
             if (migrateKeyCert(serverRoot) == false) {
                 Debug.println("ERROR AdminServerProduct.migrate: migrateKeyCert failed");
         String msg = resource.getString("status", "migrateKeyCertFailed");
         String title = resource.getString("status", "migrateTitle");
         SuiOptionPane.showMessageDialog(null, msg, title, SuiOptionPane.ERROR_MESSAGE);
                 return false;
             }
         }*/

        String msg = resource.getString("status", "migrateSucceeded");
        String title = resource.getString("status", "migrateTitle");
        SuiOptionPane.showMessageDialog(null, msg, title,
                SuiOptionPane.INFORMATION_MESSAGE);

        return true; // Success!
    }


    /**
      * Implements the IProductObject interface. Starts the Admin Server
      * creation process.
      *
      * @param  targetDN - the admin group DN where the new instance is to be
      *                    created.
      * @return  boolean value indicating whether the process succeeded (true)
      *          or failed (false).
      */
    public boolean createNewInstance(String targetDN) {
        Debug.println("trace AdminServerProduct.createNewInstance: not implemented.");
        return false;
    }


    protected boolean migrateConfig(String adminURL, String serverRoot) {

        String commandString =
                new String(adminURL + "admin-serv/tasks/operation/MigrateConfig" +
                "?oldServerRoot=" + URLByteEncoder.encodeUTF8(serverRoot));
        Debug.println(
                "TRACE AdminServerProduct.migrateConfig: MigrateConfig commandString = " +
                commandString);

        try {
            URL command = new URL(commandString);
            AdmTask task = new AdmTask(command,
                    _consoleInfo.getAuthenticationDN(),
                    _consoleInfo.getAuthenticationPassword());
            int execStatus = task.exec();
            if (execStatus != 0) {
                System.out.println("ERROR AdminServerProduct.migrateConfig: exec failed");
                return false;
            }
            int status = task.getStatus();
            if (status != 0) {
                System.out.println(
                        "ERROR AdminServerProduct.migrateConfig: exec status: " +
                        status + ": " + task.getResult("NMC_ErrInfo"));
                return false;
            }
        } catch (MalformedURLException e) {
            Debug.println(
                    "TRACE AdminServerProduct.migrateConfig: Bad URL: " + e);
            return false;
        }
        catch (Exception e) {
            Debug.println(
                    "TRACE AdminServerProduct.migrateConfig: unhandled exception: " + e);
            return false;
        }

        return true;
    }


    protected String getTargetAdminServerID(String targetDN) {
        LDAPConnection ldc = _consoleInfo.getLDAPConnection();
        if (ldc == null) {
            Debug.println("ERROR AdminServerProduct.getTargetAdminServerID: no LDAPConnection");
            return null;
        }
        String productDN = null;
        String serverDN = null;
        String serverName = null;
        try {
            LDAPEntry entry = ldc.read(targetDN);
            if (entry == null) {
                Debug.println(
                        "ERROR AdminServerProduct.getTargetAdminServerID: could not get entry = " +
                        targetDN);
                return null;
            }
            // Need to find the Fedora Administration Server product.
            LDAPAttribute attribute = entry.getAttribute("uniquemember",
                    LDAPUtil.getLDAPAttributeLocale());
            if (attribute == null) {
                Debug.println("ERROR AdminServerProduct.getTargetAdminServerID: no 'uniquemember' attribute");
                return null;
            }
            Enumeration e = attribute.getStringValues();
            while (e.hasMoreElements()) {
                productDN = (String) e.nextElement();
                if (productDN.startsWith("cn=Fedora Administration Server")) {
                    break;
                }
                productDN = null;
            }

            // Get the single admin server instance.
            if (productDN == null) {
                Debug.println("ERROR AdminServerProduct.getTargetAdminServerID: could not get product DN.");
                return null;
            }
            entry = ldc.read(productDN);
            if (entry == null) {
                Debug.println(
                        "ERROR AdminServerProduct.getTargetAdminServerID: could not get entry = " +
                        productDN);
                return null;
            }
            serverDN = LDAPUtil.flatting( entry.getAttribute("uniquemember",
                    LDAPUtil.getLDAPAttributeLocale()));
            if (serverDN != null && serverDN.length() > 3) {
                serverName = serverDN.substring(3, serverDN.indexOf(','));
            }
        }
        catch (LDAPException e) {
            Debug.println(
                    "ERROR AdminServerProduct.getTargetAdminServerID: " + e);
            return null;
        }

        return serverName;
    }


    /* protected boolean migrateKeyCert(String serverRoot) {
         String adminURL = _consoleInfo.getAdminURL();

         String adminConfigFile = _keyCert.getConfigFile();
         if (adminConfigFile == null || adminConfigFile.equals("")) {
             Debug.println("ERROR AdminServerProduct.migrateKeyCert: no admin config file");
             return false;
         }

         String newAlias = _keyCert.getNewAlias();
         if (newAlias == null || newAlias.equals("")) {
             Debug.println("ERROR AdminServerProduct.migrateKeyCert: no new key alias");
             return false;
         }

         String password = _keyCert.getPassword();
         if (password == null || password.equals("")) {
             Debug.println("ERROR AdminServerProduct.migrateKeyCert: no password");
             return false;
         }
     f
         String commandString = new String(adminURL + "admin-serv/tasks/configuration/KeyCertMigration");
     Hashtable args = new Hashtable();
     args.put("conf_file", adminConfigFile);
     args.put("newAlias", newAlias);
     args.put("password", password);
         Debug.println("TRACE AdminServerProduct.migrateKeyCert: migrate encryption commandString = " + commandString);

         try {
             URL command = new URL(commandString);
             AdmTask task = new AdmTask(command,
                                        _consoleInfo.getAuthenticationDN(),
                                        _consoleInfo.getAuthenticationPassword());
     task.setArguments(args);
             int execStatus = task.exec();
             if (execStatus != 0) {
        Debug.println("ERROR AdminServerProduct.migrateKeyCert: exec failed");
                 return false;
             }
     int status = task.getStatus();
             if (status != 0) {
        Debug.println("ERROR AdminServerProduct.migrateKeyCert: exec status: " + status + ": " + task.getResult("NMC_ErrInfo"));
                 return false;
             }
         }
         catch (MalformedURLException e) {
             Debug.println("ERROR AdminServerProduct.migrateKeyCert: Bad URL: " + e);
             return false;
         }
         catch (Exception e) {
             Debug.println("ERROR AdminServerProduct.migrateKeyCert: unhandled exception: " + e);
             return false;
         }

         Debug.println("TRACE AdminServerProduct.migrateKeyCert: migrate key cert succeeded!");
         return true;
     }*/
}
